from django.test import TestCase
from django.urls import reverse
from django.utils import timezone
from django.contrib.auth.models import User

from .models import Institution
from .models import UserRegistrationRecord
from .ldap import LDAPOperations
from .passwd import PasswordUtils

class IndexTestCase(TestCase):
    def test_index_load(self):
        response = self.client.get(reverse('user:index'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Register')
        self.assertContains(response, 'Forgot Password')


class RegistrationTestCase(TestCase):
    def setUp(self):
        self.ldap = LDAPOperations()
        self.passwd = PasswordUtils()
        self.institution = Institution.objects.create(name='Wakanda')
        self.user_password = self.passwd.getsalt(length=16) # some random chars to be the password
        self.data = {
            'first_name': 'Test',
            'last_name': 'User',
            'gender': '0',
            'email': 'sureronald@example.com',
            'title': 'Mr.',
            'designation': 'H@cker',
            'department': 'R Labs',
            'organization': self.institution.name,
            'username': 'ros1234',
            'password': self.user_password,
            'password1': self.user_password,
            'phone': '027342424234',
            'address': 'University Way, Nairobi',
            'country': 'Kenya'
        }

    def get_user_activation_code(self, username):
        user = User.objects.get(username=username)
        user_rr = UserRegistrationRecord.objects.get(user=user)
        return user_rr.reset_code

    def get_password_reset_code(self, username):
        user = User.objects.get(username=username)
        user_rr = UserRegistrationRecord.objects.get(user=user, reset_code_active=True,
                                                     reset_code_expiry__gt=timezone.now())
        return user_rr.reset_code

    def test_registration_page_load(self):
        response = self.client.get(reverse('user:register'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Create an account')

    def test_register_and_reset(self):
        response = self.client.post(reverse('user:register'), self.data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Successfully registered!')

        # test activation too
        activation_code = self.get_user_activation_code(self.data['username'])
        response = self.client.get(reverse('user:register_activate', kwargs={'activation_code': activation_code}))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Your account is now active!")

        # test password reset
        response = self.client.get(reverse('user:password_reset'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Password Reset')

        response = self.client.post(reverse('user:password_reset'), { 'email': self.data['email'] }, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Password reset sent')

        # test password edit
        reset_code = self.get_password_reset_code(self.data['username'])
        response = self.client.get(reverse('user:password_edit', kwargs={'token': reset_code}))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Enter New Password')

        new_password = self.passwd.getsalt(length=16)
        response = self.client.post(
            reverse('user:password_edit', kwargs={'token': reset_code}),
            {'password': new_password, 'password1': new_password},
            follow=True
        )
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Password edited successfully')

        # remove account so we don't have an orphan account on LDAP
        reponse = self.ldap.delete_user(self.data['username'])




from django.urls import path

from . import views

app_name = 'user'
urlpatterns = [
    path('', views.IndexView.as_view(), name='index'),
    path('register/', views.RegisterView.as_view(), name='register'),
    path('register/success/', views.RegisterSuccessView.as_view(), name='register_success'),
    path('register/activate/<slug:activation_code>/', views.RegisterActivateView.as_view(), name='register_activate'),
    path('password/reset/', views.PasswordResetView.as_view(), name='password_reset'),
    path('password/reset/success/', views.PasswordResetSuccessView.as_view(), name='password_reset_success'),
    path('password/edit/success/', views.PasswordEditSuccessView.as_view(), name='password_edit_success'),
    path('password/edit/<slug:token>/', views.PasswordEditView.as_view(), name='password_edit'),
    ]



# Generated by Django 2.0.6 on 2018-07-12 12:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0003_auto_20180710_1347'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userregistrationrecord',
            name='reset_code_expiry',
            field=models.DateTimeField(null=True),
        ),
    ]



"""
    Borrowed from the works of http://www.grotan.com/
    
    Improvements to work for Python 3 and removed some deprecated functions as well
"""
import string
import random
import base64
import hashlib
import crypt


class PasswordUtils:
    def getsalt(self, chars=string.ascii_letters + string.digits, length=16):
        """
        Generate a random salt. Default length is 16.
        Originated from mkpasswd in Luma
        
        :param chars: 
        :param length: 
        :return: string
        """
        salt = ""
        for i in range(int(length)):
            salt += random.choice(chars)
        return salt

    def mkpasswd(self, pwd, hash='ssha'):
        """
        Generate hashed passwords. Originated from mkpasswd in Luma
        
        :param pwd: 
        :param hash: 
        :return: 
        """
        alg = {
            'ssha': 'Seeded SHA-1',
            'sha': 'Secure Hash Algorithm',
            'smd5': 'Seeded MD5',
            'md5': 'MD5',
            'crypt': 'Standard unix crypt'
        }
        if hash not in alg.keys():
            return "Algorithm <%s> not supported in this version." % hash
        else:
            salt = self.getsalt()
            if hash == "ssha":
                return "{SSHA}" + base64.encodebytes(hashlib.sha1.new(str(pwd) + salt).digest() + salt)
            elif hash == "sha":
                return "{SHA}" + base64.encodebytes(hashlib.sha1.new(str(pwd)).digest())
            elif hash == "md5":
                return "{SHA}" + base64.encodebytes(hashlib.md5.new(str(pwd)).digest())
            elif hash == "smd5":
                return "{SMD%}" + base64.encodebytes(hashlib.md5.new(str(pwd) + salt).digest() + salt)
            elif hash == "crypt":
                return "{CRYPT}" + crypt.crypt(str(pwd), self.getsalt(length=2))



from django.db import models
from django.contrib.auth.models import User

class Institution(models.Model):
    name = models.CharField(max_length=255, null=True)

    def __str__(self):
        return self.name


class UserRegistrationRecord(models.Model):
    """
        much as we are sending user data to LDAP for registration, we want to keep a history
        of who has registered. We also want to use this table during password reset.
        We use django's built-in user model (User) just to keep data and avoid re-creating fields
        that Django already provides. Note that we are not setting the password fields. We are not
        even keeping the passwords. They belong to LDAP!
    """
    # controls
    reset_code = models.CharField(max_length=255, default='') # used to hold activation code for registration too
    reset_code_active = models.BooleanField(default=False)
    reset_code_expiry = models.DateTimeField(null=True)

    # user data
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    ldap_password = models.CharField(max_length=128, blank=True, null=True)
    gender = models.CharField(max_length=10, blank=True)
    title = models.CharField(max_length=30, blank=True)
    designation = models.CharField(max_length=200, blank=True)
    department = models.CharField(max_length=200, blank=True)
    organization = models.CharField(max_length=255, blank=True, null=True)
    phone = models.CharField(max_length=200, blank=True)
    address = models.TextField(blank=True)
    country = models.CharField(max_length=100, blank=True, null=True)

    def __str__(self):
        return self.user.username



# Generated by Django 2.0.6 on 2018-07-10 13:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0002_userregistrationrecord'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userregistrationrecord',
            name='address',
            field=models.TextField(blank=True),
        ),
        migrations.AlterField(
            model_name='userregistrationrecord',
            name='country',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AlterField(
            model_name='userregistrationrecord',
            name='designation',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AlterField(
            model_name='userregistrationrecord',
            name='gender',
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AlterField(
            model_name='userregistrationrecord',
            name='organization',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='userregistrationrecord',
            name='phone',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AlterField(
            model_name='userregistrationrecord',
            name='reset_code_expiry',
            field=models.DateTimeField(blank=True),
        ),
        migrations.AlterField(
            model_name='userregistrationrecord',
            name='title',
            field=models.CharField(blank=True, max_length=30),
        ),
    ]



# Utility functions

from django.core.mail import send_mail


def send_reset_password_email(email, from_email, full_name, reset_link, app_name):
    """
    Send password reset email
    :param email:
    :param from_email:
    :param full_name: 
    :param reset_link: 
    :param app_name: 
    :return: 
    """
    tpl = """

Hello {full_name},

Someone has requested a link to change your password. You can do this through the link below:

{reset_link}

If you didn't request this, please ignore this email.

Your password won't change until you access the link above and create a new one.

-- {app_name}
"""
    subject = '[' + app_name + '] Password reset instructions'
    msg_map = { 'full_name': full_name, 'reset_link': reset_link, 'app_name': app_name }
    message = tpl.format_map(msg_map)

    send_mail(subject, message, from_email, [email])


def send_newly_registered_email(email, from_email, full_name, activate_link, app_name):
    """
    Send notification for new registrations with activation link
    :param email: 
    :param from_email: 
    :param full_name: 
    :param activate_link: 
    :param app_name: 
    :return: 
    """
    tpl = """

Welcome {full_name}!

The first step is to verify your email address. Please click the link or paste the URL into your web browser:

{activate_link}

-- {app_name}
"""
    subject = 'Welcome to ' + app_name + '!'
    msg_map = { 'full_name': full_name, 'activate_link': activate_link, 'app_name': app_name }
    message = tpl.format_map(msg_map)

    send_mail(subject, message, from_email, [email])



from datetime import timedelta

from django.views import generic
from django.contrib.auth.models import User
from django.conf import settings
from django.shortcuts import render, redirect
from django.utils import timezone

from .forms import UserRegisterForm
from .forms import PasswordResetForm
from .forms import PasswordResetEditForm
from .models import UserRegistrationRecord
from .ldap import LDAPOperations
from .passwd import PasswordUtils
from .utils import send_reset_password_email
from .utils import send_newly_registered_email
from .exceptions import AccountActivationException
from .exceptions import PasswordResetException


class IndexView(generic.TemplateView):
    # Index View
    template_name = 'user/index.html'


class RegisterView(generic.FormView):
    template_name = 'user/register.html'
    form_class = UserRegisterForm
    success_url = '/user/register/success/'

    def form_valid(self, form):
        passwd_util = PasswordUtils()

        data = form.cleaned_data
        password = passwd_util.mkpasswd(data.get('password'), hash='crypt')

        token = passwd_util.getsalt(length=60)

        # we want to keep record of successful registrations
        user = User.objects.create_user(
            username=data.get('username'),
            email=data.get('email'),
            first_name=data.get('first_name'),
            last_name=data.get('last_name'),
            is_active=False,
        )
        # We keep the user data for later when the user activates the account so we can create it on
        # the LDAP back-end
        user_record = UserRegistrationRecord.objects.create(
            user=user,
            ldap_password=password,
            reset_code=token,
            gender=data.get('gender'),
            title=data.get('title'),
            designation=data.get('designation'),
            department=data.get('department'),
            organization=data.get('organization'),
            phone=data.get('phone'),
            address=data.get('address'),
            country=data.get('country')
        )

        # send notification and activation email alert
        activate_link = settings.SITE_BASE_URL + '/user/register/activate/' + token + '/'

        send_newly_registered_email(user.email, settings.DEFAULT_FROM_EMAIL,
                                  user.get_full_name(), activate_link, settings.IDP_NAME)

        return super().form_valid(form)


class RegisterSuccessView(generic.TemplateView):
    template_name = 'user/register_success.html'


class RegisterActivateView(generic.View):
    def get(self, request, activation_code):
        ldap_ops = LDAPOperations()
        uid_number = str(self.generate_uid_number())
        try:
            user_rr = UserRegistrationRecord.objects.get(reset_code=activation_code)
        except UserRegistrationRecord.DoesNotExist:
            raise AccountActivationException('Invalid activation code')
        user = user_rr.user
        if user.is_active:
            raise AccountActivationException('This account is already active')

        modlist = {
            "objectClass": ["inetOrgPerson", "posixAccount", "shadowAccount"],
            "uid": [user.username],
            "userPassword": [user_rr.ldap_password],
            "sn": [user.last_name],
            "givenName": [user.first_name],
            "cn": [user.get_full_name()],
            "displayName": [user.get_full_name()],
            "title": [user_rr.title],
            "mail": [user.email],
            "employeeType": [user_rr.designation],
            "departmentNumber": [user_rr.department],
            "telephoneNumber": [user_rr.phone],
            "registeredAddress": [user_rr.address],
            "homePhone": [user_rr.phone],
            "uidNumber": [uid_number],
            "gidNumber": [settings.LDAP_GID],
            "loginShell": ["/bin/bash"],
            "homeDirectory": ["/home/users/" + user.username]
        }
        # if this is a CATCH ALL IDP, we need to capture more user attributes since it serves many
        # user organizations and possibly different countries
        if settings.IDP_CATCH_ALL:
            modlist['description'] = ['Organization: ' + user_rr.organization.name,
                                      'Country: ' + user_rr.country]

        result = ldap_ops.add_user(modlist)
        if result:
            # we need to de-activate the code so it can't be re-used and also activate user account just
            # for record purposes since we are not using django's login mechanism anyway
            user_rr.reset_code = ''
            user_rr.save()
            user.is_active = True
            user.save()

        return render(request, 'user/register_activate_success.html', {
            'result': result,
            'message': "Your account is now active!",
        })

    def generate_uid_number(self):
        """
        Find the last record of user. Get UID base and increment by adding the last record pk
        (primary key) and one
        :return: int
        """
        uid_number = settings.LDAP_BASE_UID
        try:
            latest = User.objects.latest('pk')
            uid_number += latest.pk + 1
        except User.DoesNotExist:
            pass

        return uid_number


class PasswordResetView(generic.FormView):
    template_name = 'user/password_reset.html'
    form_class = PasswordResetForm
    success_url = '/user/password/reset/success/'

    def form_valid(self, form):
        """
        1. get user by the provided email from db (die silently if there's none)
        2. if exists and is active, send reset email
        :param form: 
        :return: 
        """
        passwd_util = PasswordUtils()
        email = form.cleaned_data.get('email')
        user = User.objects.get(email=email)
        user_reg_record = UserRegistrationRecord.objects.get(user=user)
        token = passwd_util.getsalt(length=60) # re-use salt method to generate unique token
        expiry = timezone.now() + timedelta(hours=settings.PASSWORD_RESET_TOKEN_EXPIRY)
        user_reg_record.reset_code = token
        user_reg_record.reset_code_active = True
        user_reg_record.reset_code_expiry = expiry
        user_reg_record.save()

        reset_link = settings.SITE_BASE_URL + '/user/password/edit/' + token + '/'
        # send email
        send_reset_password_email(user.email, settings.DEFAULT_FROM_EMAIL,
                                  user.get_full_name(), reset_link, settings.IDP_NAME)

        return super().form_valid(form)


class PasswordResetSuccessView(generic.TemplateView):
    template_name = 'user/password_reset_success.html'


class PasswordEditView(generic.View):
    template_name = 'user/password_edit.html'

    def get(self, request, token):
        self.check_token_validity(token)
        return render(request, self.template_name, {
            'form': PasswordResetEditForm(),
        })

    def post(self, request, token):
        ldap_ops = LDAPOperations()
        passwd_util = PasswordUtils()
        user_rr = self.check_token_validity(token)

        form = PasswordResetEditForm(request.POST)
        if form.is_valid():
            data = form.cleaned_data
            username = user_rr.user.username
            password = passwd_util.mkpasswd(data.get('password'), hash='crypt')

            result = ldap_ops.set_password(username, password)
            if result:
                user_rr.reset_code_expiry = timezone.now()
                user_rr.reset_code = ''
                user_rr.reset_code_active = False
                user_rr.ldap_password = password
                user_rr.save()
                return redirect('/user/password/edit/success/')
            else:
                return render(request, self.template_name, {
                    'form': form,
                    'ldap_error': 'Password reset failed on LDAP server. Contact admin'
                })
        else:
            return render(request, self.template_name, {
                'form': form,
            })

    def check_token_validity(self,token):
        try:
            user_rr = UserRegistrationRecord.objects.get(reset_code=token,
                                                         reset_code_active=True,
                                                         reset_code_expiry__gt=timezone.now())
        except UserRegistrationRecord.DoesNotExist:
            raise PasswordResetException('Password reset link invalid or expired')
        return user_rr


class PasswordEditSuccessView(generic.TemplateView):
    template_name = 'user/password_edit_success.html'



#Custom context processors

from django.conf import settings


def site(request):
    return {
        'SITE_BASE_URL': settings.SITE_BASE_URL,
        'IDP_NAME': settings.IDP_NAME,
        'IDP_LOGO': settings.IDP_LOGO,
        'SERVICE_PROVIDER': settings.SERVICE_PROVIDER,
        'SERVICE_PROVIDER_URL': settings.SERVICE_PROVIDER_URL
    }


"""
    LDAP Operations:
"""
import ldap
from ldap.modlist import addModlist

from django.conf import settings


class LDAPOperations():
    def __init__(self):
        self.connect()

    def connect(self):
        if settings.LDAP_PROTO == 'ldaps':
            ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)
        self.con = ldap.initialize(settings.LDAP_PROTO + '://' + settings.LDAP_HOST + ':' + settings.LDAP_PORT)
        try:
            self.con.simple_bind_s(settings.LDAP_BIND_DN, settings.LDAP_BIND_DN_CREDENTIAL)
        except ldap.SERVER_DOWN:
            raise ldap.SERVER_DOWN('The LDAP library can’t contact the LDAP server. Contact the admin.')

    def check_attribute(self,attribute,value):
        """
        Takes an attribute and value and checks it against the LDAP server for existence/availability.
        This is mainly for checking unique attributes ie uid, mail, uidNumber
        
        :param attribute:
        :param value
        :return: tuple
        """
        query = "("+ attribute + "=" + value + ")"
        result = self.con.search_s(settings.LDAP_BASE_DN, ldap.SCOPE_SUBTREE, query)
        return result

    def add_user(self, modlist):
        """
        example param modlist dict should look like below containing only strings:
        
        modlist = {
            "objectClass": ["inetOrgPerson", "posixAccount", "shadowAccount"],
            "uid": ["jdoe"],
            "sn": ["Doe"],
            "givenName": ["John"],
            "cn": ["John Doe"],
            "displayName": ["John Doe"],
            "mail": ["jdoe@example.com"],
            "homePhone": ["07234232434"],
            "uidNumber": ["1003"], # generate a unique ID
            "gidNumber": [502], # get from settings.LDAP_GID
            "loginShell": ["/bin/bash"],
            "homeDirectory": ["/home/users/jdoe"]
        }
        
        :param modlist: 
        :return: tuple
        
        """
        dn = 'uid=' + modlist['uid'][0] + ',' + settings.LDAP_BASE_DN

        # convert modlist to bytes form ie b'abc'
        modlist_bytes = {}
        for key in modlist.keys():
            modlist_bytes[key] = [i.encode('utf-8') for i in modlist[key]]

        result = self.con.add_s(dn, addModlist(modlist_bytes))
        return result

    def set_password(self,username,password):
        """
        set user password
        :param username: 
        :param password: 
        :return: ldap result
        """
        dn = "uid=%s,%s" % (username,settings.LDAP_BASE_DN,)
        user_result = self.check_attribute('uid', username)  # get user
        tmp_modlist = dict(user_result)
        old_value = {"userPassword": [tmp_modlist[dn]['userPassword'][0]]}
        new_value = {"userPassword": [password.encode()]}

        modlist = ldap.modlist.modifyModlist(old_value, new_value)
        result = self.con.modify_s(dn, modlist)
        return result

    def delete_user(self,username):
        dn = "uid=%s,%s" % (username, settings.LDAP_BASE_DN,)
        response = self.con.delete_s(dn)
        return response



from django import forms
from django.contrib.auth.validators import UnicodeUsernameValidator
from django.contrib.auth.models import User

from crispy_forms.bootstrap import Field, InlineRadios, TabHolder, Tab
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit, Layout, Div, Fieldset

from .models import Institution
from .ldap import LDAPOperations


class UserRegisterForm(forms.Form):
    title_choices = (('Mr.', 'Mr'), ('Ms', 'Ms',), ('Mrs.', 'Mrs.',), ('Dr.', 'Dr.',), ('Prof.', 'Prof.',),)
    country_choices = (('Kenya', 'Kenya'),)

    first_name = forms.CharField(required=True, max_length=255)
    last_name = forms.CharField(required=True, max_length=255)
    gender = forms.TypedChoiceField(
        choices=((0, "Male"), (1, "Female"),),
        coerce=lambda x: bool(int(x)),
        widget=forms.RadioSelect,
        initial='0',
        required=True)
    email = forms.EmailField(required=True)
    title = forms.ChoiceField(required=True, choices=title_choices)
    designation = forms.CharField(max_length=200)
    department = forms.CharField(required=True, max_length=255)
    organization = forms.ModelChoiceField(required=False, queryset=Institution.objects.all(),
                                          empty_label='Select an organization', to_field_name='name')
    username = forms.CharField(required=True, min_length=3, max_length=30, help_text='Choose a memorable name e.g jdoe',
                               validators=[UnicodeUsernameValidator()])
    password = forms.CharField(widget=forms.PasswordInput, min_length=8)
    password1 = forms.CharField(widget=forms.PasswordInput, min_length=8, label='Confirm Password')
    phone = forms.CharField(required=True, max_length=200)
    address = forms.CharField(max_length=1000, widget=forms.Textarea())
    country = forms.ChoiceField(required=False, choices=country_choices)  # Set to default to your country

    def __init__(self, *args, **kwargs):
        super(UserRegisterForm, self).__init__(*args, **kwargs)
        self.ldap_ops = LDAPOperations()
        self.helper = FormHelper()
        self.helper.form_id = 'id-user-data-form'
        self.helper.form_method = 'post'
        # self.helper.form_action = 'register'
        self.helper.add_input(Submit('submit', 'Submit', css_class='btn-success'))
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2'
        self.helper.field_class = 'col-md-8'
        self.helper.error_text_inline = False
        self.helper.layout = Layout(
            Fieldset('Personal Data',
                     Field('first_name', placeholder='Your first name',
                           css_class="some-class"),
                     Div('last_name', title="Your last name"),
                     'email', 'phone', 'gender', 'title', 'designation', 'department', 'organization',),
            Fieldset('Login Details', 'username', 'password', 'password1',),
            Fieldset('Address', 'address', 'country',))

    def clean_username(self):
        username = self.cleaned_data['username']
        # check username existence in local storage DB
        query_set = User.objects.filter(username=username)

        # check username existence in LDAP

        result = self.ldap_ops.check_attribute('uid', username)
        if result or query_set:
            raise forms.ValidationError("Username " + username + " is not available (in use)",
                                        code='username_exists_ldap')
        return username

    def clean_email(self):
        mail = self.cleaned_data['email']
        # check for email existence in local storage DB
        query_set = User.objects.filter(email=mail)

        # check email existence in LDAP
        result = self.ldap_ops.check_attribute('mail', mail)
        if result or query_set:
            raise forms.ValidationError("Email " + mail + " is not available (in use)",
                                        code='email_exists_ldap')
        return mail

    def clean(self):

        # Check for password matching

        password = self.cleaned_data.get('password')
        password1 = self.cleaned_data.get('password1')

        if password != password1:
            self._errors["password"] = self.error_class(["Passwords do not match"])

        return self.cleaned_data


class PasswordResetForm(forms.Form):
    email = forms.EmailField(
        required=True,
        label='ENTER YOUR EMAIL',
        help_text="An email will be sent to the address you specify, containing a link that will allow you to change\
         your old password. "
    )

    def __init__(self, *args, **kwargs):
        super(PasswordResetForm, self).__init__(*args, **kwargs)
        self.ldap_ops = LDAPOperations()
        self.helper = FormHelper()
        self.helper.form_id = 'id-password-reset-form'
        self.helper.form_method = 'post'
        self.helper.add_input(Submit('submit', 'Submit', css_class='btn-warning'))
        #self.helper.form_class = 'form-horizontal'
        #self.helper.label_class = 'col-md-2'
        #self.helper.field_class = 'col-md-8'
        self.helper.error_text_inline = False
        self.helper.layout = Layout(
            Field('email', placeholder='Your E-mail')
        )

    def clean_email(self):
        mail = self.cleaned_data['email']
        # check for email existence in local storage DB
        query_set = User.objects.filter(email=mail, is_active=True)

        # check email existence in LDAP
        result = self.ldap_ops.check_attribute('mail', mail)
        if not result and not query_set:
            raise forms.ValidationError("This email address doesn't have an associated user account. \
            Please make sure you have registered, before proceeding.",
                                        code='email_exists_ldap')
        return mail


class PasswordResetEditForm(forms.Form):
    password = forms.CharField(widget=forms.PasswordInput, min_length=8, label='New password')
    password1 = forms.CharField(widget=forms.PasswordInput, min_length=8, label='New password confirmation')

    def __init__(self, *args, **kwargs):
        super(PasswordResetEditForm, self).__init__(*args, **kwargs)
        self.ldap_ops = LDAPOperations()
        self.helper = FormHelper()
        self.helper.form_id = 'id-password-reset-edit-form'
        self.helper.form_method = 'post'
        self.helper.add_input(Submit('submit', 'Submit', css_class='btn-warning'))
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-4'
        self.helper.field_class = 'col-md-8'
        self.helper.error_text_inline = False
        self.helper.layout = Layout(
            Fieldset('Please enter your new password',
                     Field('password'),
                     Field('password1')
                     )
        )

    def clean(self):
        # Check for password matching

        password = self.cleaned_data.get('password')
        password1 = self.cleaned_data.get('password1')

        if password != password1:
            self._errors["password"] = self.error_class(["Passwords do not match"])

        return self.cleaned_data



# Generated by Django 2.0.6 on 2018-07-10 08:11

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('user', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserRegistrationRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reset_code', models.CharField(default='', max_length=255)),
                ('reset_code_active', models.BooleanField(default=False)),
                ('reset_code_expiry', models.DateTimeField()),
                ('gender', models.CharField(max_length=10)),
                ('title', models.CharField(max_length=30)),
                ('designation', models.CharField(max_length=200)),
                ('organization', models.CharField(max_length=255)),
                ('phone', models.CharField(max_length=200)),
                ('address', models.TextField()),
                ('country', models.CharField(max_length=100)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

standard_exclude_directories = [
    ".*", "./django_ldap_user_registration/*", "./build",
    "./dist", "EGG-INFO", "*.egg-info",
]
# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-ldap-user-registration',
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    packages=find_packages(exclude=standard_exclude_directories),
    include_package_data=True,
    license='GPL v3',  # example license
    description='A Django web front-end that provides user registration and password reset to an LDAP server.',
    long_description=README,
    url='https://github.com/KENET-KE/django-ldap-user-registration/',
    author='Ronald Osure',
    author_email='sureronald@gmail.com',
    keywords ='django ldap registration password reset idp federation catch-all',
    install_requires=[
        'Django >= 2.0',
        'python-ldap',
        'django-bootstrap3',
        'django-anymail',
        'django-crispy-forms',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# Generated by Django 2.0.6 on 2018-07-17 09:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0005_auto_20180712_1308'),
    ]

    operations = [
        migrations.AddField(
            model_name='userregistrationrecord',
            name='ldap_password',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
    ]



# Generated by Django 2.0.6 on 2018-07-12 13:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0004_auto_20180712_1254'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userregistrationrecord',
            name='country',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='userregistrationrecord',
            name='organization',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
